unit fedutils;

interface

uses forms, winprocs, wintypes, classes, sysutils;

Function MyStrLen(Mem : THandle) : longint;
Function MakePath(Path : String) : String;
procedure Split(d, value : string; var s1, s2 : string);
Function SysMinimized(Form : TForm) : Boolean;
Function NiceClassname(TheClass : TClass) : String;
Function GetBestBitMap(TheClass : TClass) : HBitMap;
function FileCreateNoTrunc(const FileName: string): Integer;
Function FileCreateShare(const Filename : String; Mode : Word) : Integer;
Function RemoveFileExtention(const Filename : TFilename) : TFilename;
Function OpenResource(ResourceName : String; var Size : longint) : Integer;
Function CopyResourceToFile(ResourceName : String; var UserFileHandle : Integer) : TFilename;
Function GetTempName : TFilename;
Function FileEOF(Handle : Integer) : Boolean;

implementation

type
  TFileNameBuf = array[0..79] of Char;

Function getBestBitMap(TheClass : TClass) : HBitMap;
var
  Buffer : Array[0..255] of char;
  ParentClass : TClass;
begin
  Result:=LoadBitmap(hInstance,
            StrPCopy(Buffer,UpperCase(TheClass.Classname)));
  ParentClass:=TheClass;
  While (result=0) do begin
    ParentClass:=ParentClass.ClassParent;
    If not ParentClass.InheritsFrom(TComponent) then break;
    Result:=LoadBitmap(hInstance,
            StrPCopy(Buffer,UpperCase(ParentClass.Classname)));
  end;
end;

Function MyStrLen;
var
  Data : PChar;
  Offset, Size : longint;
  flags : word;
  WOffset : Word;
begin
  try
{    flags:=globalflags(mem);}
    Size:=GlobalSize(mem);
    Data:=GlobalLock(mem);
    Offset:=0;
    If Data=nil then begin
      result:=0;
      exit;
    end;
    while true do begin
      Result:=StrLen(Data);
      If Data[Result]<>#0 then begin { lied!!!! }
        WOffset:=ofs(Data^)+Result;
        If WOffset>=65534 then begin
          Data:=Ptr(SelectorInc+Seg(Data^),0);
          Inc(Offset,65536);
        end else begin
          { why stop here, its not the end of the segment? }
          Raise Exception.Create('MyStrLen stopped in middle of segment');
        end;
      end else { ok!.... } begin
        Result:=Result+Offset;
        If Result>Size then begin
          Result:=Size;
        end;
        break;
      end;
      If Offset>Size then begin { must be running away with ourselves }
        Result:=Size;
        break;
      end;
    end;
  finally
    GlobalUnlock(Mem);
  end;
end;

Function MakePath;
begin
  If Path='' then
    result:='.\'
  else begin
    if Path[length(Path)]='\' then
      result:=Path
    else
      Result:=Path+'\';
  end;
end;

procedure Split(d, value : string; var s1, s2 : string);
var
  p : integer;
begin
  p:=pos(d,value);
  If p=0 then p:=length(Value)+1;
  s1:=copy(value,1,p-1);
  s2:=copy(value,p+1,255);
end;

Function SysMinimized(Form : TForm) : Boolean;
var
  Placement : TWindowPlacement;
begin
  Result:=False;
  Placement.length:=sizeof(Placement);
  GetWindowPlacement(Form.handle,@Placement);
  Result:=Placement.Showcmd=SW_ShowMinimized;
end;

Function NiceClassName;
var
  c : integer;
begin
  Result:=TheClass.Classname;
  { skip all capital letters until last one in first series }
  c:=1; while c<=length(Result) do begin
    If Result[c]='_' then begin
      inc(c,2);
      break; { return after the _ }
    end;
    if not ((Result[c]>='A') and (Result[c]<='Z')) then break;
    inc(c);
  end;
  If C>=length(result) then exit;
  Result:=Copy(Result,c-1,255);
end;

procedure CopyFileName(Dest: PChar; const Source: string);
begin
  AnsiToOem(StrPLCopy(Dest, Source, SizeOf(TFileNameBuf) - 1), Dest);
end;

function FileCreateNoTrunc(const FileName: string): Integer; assembler;
var
  FileNameBuf: TFileNameBuf;
asm
        LEA     DI,FileNameBuf
        PUSH    SS
        PUSH    DI
        PUSH    FileName.Word[2]
        PUSH    FileName.Word[0]
        CALL    CopyFileName
        PUSH    DS
        LEA     DX,FileNameBuf
        PUSH    SS
        POP     DS
        XOR     CX,CX
        MOV     AH,5BH
        INT     21H
        POP     DS
        JNC     @@1
        NEG     AX
@@1:
end;

Function FileCreateShare(const Filename : String; Mode : Word) : Integer;
begin
  Result:=FileCreateNoTrunc(Filename);
  If Result>=0 then { didn't exist, does now, ok.... }
    FileClose(Result); { cos we want to specify share permissions }
  { now open it!!! }
  Result:=FileOpen(Filename, Mode);
  { now truncate it }
  FileSeek(Result,0,0);
  FileWrite(Result,pointer(nil)^,0);
end;

Function RemoveFileExtention;
var
  l, c : Integer;
begin
  Result:=Filename;
  l:=length(filename);
  c:=l; while (c>0) and (c>(l-4)) do begin
    if Filename[c]='.' then begin
      Result:=copy(Filename,1,c-1);
    end;
    dec(c);
  end;
end;

Function OpenResource(ResourceName : String; var Size : longint) : Integer;
var
  HResInfo: THandle;
  Stream: TStream;
  Name: array[0..63] of Char;
begin
  StrPLCopy(Name, ResourceName, SizeOf(Name) - 1);
  HResInfo := FindResource(HInstance, Name, rt_RCData);
  if HResInfo = 0 then
    raise EResNotFound.Create(ResourceName+' not found');
  If @size<>nil then
    Size:=SizeOfResource(HInstance, HResInfo);
  Result := AccessResource(HInstance, HResInfo);
  if Result < 0 then
    raise EResNotFound.Create(ResourceName+' not found');
end;

Function CopyResourceToFile(ResourceName : String; var UserFileHandle : Integer) : TFilename;
const BufSize=4096;
var
  ResHandle, FileHandle : Integer;
  endres : longint;
  Buffer : Pointer;
  BlockSize : longint;
begin
  Result:=GetTempname;
  ResHandle:=OpenResource(ResourceName,endres);
  FileHandle:=FileOpen(Result,fmOpenReadWrite or fmShareExclusive);
  GetMem(Buffer,BufSize);
  BlockSize:=BufSize;
  try
    While Endres>0 do begin
      If EndRes<BlockSize then BlockSize:=EndRes;
      FileRead(ResHandle,Buffer^,BlockSize);
      FileWrite(FileHandle,Buffer^,BlockSize);
      EndRes:=EndRes-BlockSize;
    end;
  finally
    FreeMem(Buffer,BufSize);
  end;
  If @UserFileHandle<>nil then begin
    userfileHandle:=FileHandle;
    FileSeek(FileHandle,0,0);
  end else
    FileClose(FileHandle);
  FileClose(ResHandle);
end;

Function GetTempName;
var
  Buffer : Array[0..255] of char;
begin
  GetTempFileName(GetTempDrive(#0),'',0,Buffer);
  Result:=StrPas(Buffer);
end;

Function FileEof;
var
  Pos : Longint;
  Size : Longint;
begin
  Pos:=FileSeek(Handle,0,1);
  Size:=FileSeek(Handle,0,2);
  FileSeek(Handle,Pos,0);
  Result:=pos=size;
end;

end.
