unit Holder;

{ This unit defines a control to represent a component on the screen }
{ Otherwise known as a HOLDER }
interface

uses buttons, classes, dsgnintf, winprocs, wintypes, sysutils, messages, forms, fedutils;

const
  PWM_ComponentMoved : Word=0;

Type
  TComponentHolder=class(TBitBtn) { cos it already has the right look }
  protected
    FCreated : Boolean;
    FComponent : TComponent;    { The component we are faking }
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
    Procedure SetComponent(Component : TComponent);
    Procedure WndProc(Var Message : TMessage); override;
  public
    Procedure MatchComponent;
    Constructor Create(AOwner : TComponent); Override;
    Destructor Destroy; override;
    Procedure FreeComponent;
    Procedure ComponentMoved;
    Property Component : TCOmponent Read FComponent Write SetComponent;
  end;

  TComponentHolderEditor=class(TComponentEditor) { Ask for an editor for the component }
  protected                                      { and then hard link methods }
    FComponentEditor : TComponentEditor;
  public
    Constructor Create(AComponent: TComponent; ADesigner: TFormDesigner); override;
    Destructor Destroy;
    Procedure Copy; Override;
    Function GetVerb(Index : Integer) : String; override;
    Function GetVerbCount : Integer; Override;
    Procedure ExecuteVerb(Index : Integer); override;
    Procedure Edit; Override;
  end;

implementation

Constructor TComponentHolder.Create;
begin
  Inherited Create(AOwner);
  UpdateBoundsRect(Rect(0,0,28,28));
  { Now force it to use these values }
  Left:=0;
  FCreated:=True;
end;

Procedure TComponentHolder.FreeComponent;
begin
  If FComponent<>nil then begin
    FComponent.Owner.RemoveComponent(FComponent);
    FComponent.Free;
    FComponent:=Nil;
  end;
end;

Procedure TComponentHolder.SetBounds;
begin
  If (Left<>ALeft) or (Top<>ATop) or (Width<>AWidth) or (Height<>AHeight) then begin
    If FCreated then begin
      Inherited SetBounds(ALeft, ATop, Width, Height);
      If FComponent<>nil then
        FComponent.DesignInfo:=makelong(ATop,ALeft);
    end else
      Inherited SetBounds(ALeft,ATop,Width,Height);
  end;
end;

Procedure TComponentHolder.SetComponent;
var
  L, T : Integer;

begin
  FComponent:=Component;
  L:=HiWord(FComponent.DesignInfo);
  T:=LoWord(FComponent.DesignInfo);
  SetBounds(L,T,Width,Height);
  Glyph.Releasehandle;
  Glyph.Handle:=GetBestBitMap(Component.classtype);
end;

Constructor TComponentHolderEditor.Create;
var
  Holder : TComponentHolder;
begin
  Inherited Create(AComponent, ADesigner);
  { This should always be as this Component editor should only every }
  { Be returned for TComponentHolder }
  Holder:=AComponent as TComponentHolder;
  { A Holder should ALWAYS have a component }
  FComponentEditor:=GetComponentEditor(Holder.Component,ADesigner);
end;

Destructor TComponentHolderEditor.Destroy;
begin
  FComponentEditor.Free;
  Inherited Destroy;
end;

Procedure TComponentHolderEditor.Copy;
begin
  If FComponentEditor<>nil then FComponenteditor.Copy;
end;

Procedure TComponentHolderEditor.Edit;
begin
  If FComponentEditor<>nil then FComponenteditor.Edit;
end;

Function TComponentHolderEditor.GetVerb;
begin
  If FComponentEditor<>nil then
    Result:=FComponenteditor.GetVerb(Index);
end;

Procedure TComponentHolderEditor.ExecuteVerb;
begin
  If FComponentEditor<>nil then
    FComponenteditor.ExecuteVerb(Index);
end;

Function TComponentHolderEditor.GetVerbCount;
begin
  If FComponentEditor<>nil then
    Result:=FComponenteditor.GetVerbCount
  else
    Result:=0;
end;

Procedure TComponentHolder.WndProc;
var
  Form : TForm;
begin
  { we have to do this as we can't be in design mode ourself, our our hint won't work }
  Form := GetParentForm(Self);
  if (Form <> nil) and (Form.Designer <> nil) and
    Form.Designer.IsDesignMsg(Self, Message) then Exit;

  If Message.Msg=PWM_ComponentMoved then begin
    ComponentMoved;
  end else begin
    Inherited WndProc(Message);
  end;
end;

Procedure TComponentHolder.MatchComponent;
begin
  If (Component.DesignInfo)<>MakeLong(Top,Left) then
    PostMessage(Handle,PWM_ComponentMoved,0,0);
end;

Procedure TComponentHolder.ComponentMoved;
begin
  SetBounds(HiWord(Component.DesignInfo),LoWord(Component.DesignInfo),0,0); { last two are overridden }
end;

Destructor TComponentHolder.Destroy;
begin
  FComponent:=nil;
  Inherited Destroy;
end;

begin
  RegisterComponentEditor(TComponentHolder,TComponentHoldereditor);
  PWM_ComponentMoved:=RegisterWindowMessage('PWM_ComponentMoved');
end.


