unit Insp2old;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, dsgnintf, peedit, Tabs, ExtCtrls, Buttons,
  typinfo, mtbase, inifiles,peedit2, fedutils;

type
  TLoadInspector=class;

  TInspector = class(TForm)
    TabSet1: TTabSet;
    Panel1: TPanel;
    CBOnTop: TCheckBox;
    PropertyPanel: TPanel;
    Panel3: TPanel;
    Notebook1: TNotebook;
    Label1: TLabel;
    LCurrentForm: TLabel;
    Label3: TLabel;
    LCurrentControl: TLabel;
    InspHelpButton: TBitBtn;
    procedure TabSet1Change(Sender: TObject; NewTab: Integer;
      var AllowChange: Boolean);
    procedure FormShow(Sender: TObject);
    procedure OKButtonClick(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure CheckBox2Click(Sender: TObject);
    procedure ShowHelpCBClick(Sender: TObject);
    procedure CBOnTopClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure InspHelpButtonClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
    Procedure RefreshProportions;
  Protected
    FDesigner : TFormDesigner;
    FSelectedComponents : TList;
    FKeyComponent : TComponent;
    FMapPages : TStringlist;
    FHints : TStringlist;
    Procedure SetCurrentForm(Form : TForm);
    Function GetKeyComponent : TComponent;
    Procedure PositionPropertyEditor(PageIndex : Integer; PropertyLabel : TLabel; VPropertyEditor : TVpe;
              PropertyEditor : TPropertyEditor);
    Function GetShowWidth : Integer;
    Function GetActiveForm : TForm;
    Procedure RemoveBlankPages;
    Procedure CommitEdit(Commit : Boolean); { if any peedit has focus and uncomiited changes, abort them }
  public
    { Public declarations }
    Constructor Create(AOwner : TComponent); override;
    Destructor Destroy; override;
    Procedure CommitChanges(Designer : TFormDesigner; Commit : Boolean);
    Procedure Clear;
    procedure Prepare(ComponentList : TList);
    procedure AddProperty(PropertyEditor: TPropertyEditor);
    procedure SetupHint(Sender : TObject);
    Procedure UseComponents(Components : TList; Designer : TformDesigner);
    Procedure SetFocusPropertyEditor(Sender : TObject);
    Procedure Renew(Designer : TFormDesigner);
    Property keyComponent : TComponent Read GetKeyComponent;
    Procedure HideHelp;
    Procedure ShowHelp;
    Procedure SelectNextForm(OldForm : TForm);
    Property ActiveEditor : TFormDesigner Read Fdesigner;
    Property ActiveForm : TForm Read GetActiveForm;
  end;

  TLiTask=(liNothing,liLoad,liRenew);

  TLoadInspector=class(TThread)
  public
    FTask : TLiTask;
    FCreator : Pointer;
    FDesigner : TFormDesigner;
    FInspector : TInspector;
    FComponents : TList;
    FNeedsRenew : Boolean;
    Procedure Run; override;
    procedure AddProperty(PropertyEditor: TPropertyEditor);
    Procedure Load;
    Procedure Renew;
  end;

var
  Inspector: TInspector;

const
  LoadInspector : TLoadInspector=nil;

implementation

{$R *.DFM}

uses fedit2, main;

Constructor TInspector.Create;
begin
  Inherited Create(AOwner);
  { Find path }
  If LoadInspector=nil then begin
    LoadInspector:=TLoadInspector.Create(0,tsApplicationAuto,8192);
    LoadInspector.FInspector:=Self;
    Loadinspector.FCreator:=Self;
  end;
  FMapPages:=TStringList.Create;
  FHints:=TStringList.Create;
  Icon.Handle:=LoadIcon(hInstance,'INSPECTORICON');
end;

Destructor TInspector.Destroy;
begin
  If LoadInspector<>nil then
    If loadInspector.FCreator=self then
      LoadInspector.Free;
  FMapPages.Free;
  FHints.Free;
  Inherited Destroy;
end;

Function TInspector.GetKeyComponent;
begin
  Result:=FKeyComponent;
end;

procedure TInspector.CommitEdit;
var
  Control : TControl;
begin
  If (ActiveControl<>nil) and (ActiveControl is TControl) then begin
    Control:=ActiveControl as TControl;
    While Control<>Self do begin
      If Control is TCustomVisualPropertyEditor then begin
        (Control as TCustomVisualPropertyEditor).CommitChange(Commit);
        break;
      end;
      Control:=Control.Parent;
    end;
  end;
end;

procedure TInspector.AddProperty(PropertyEditor: TPropertyEditor);
var
  PropertyLabel : TLabel;
  VPropertyEditor : TVpe;
  Propertyname, Pagename : String;
  Index : Integer;
begin
  { Only add this property if it is index, and add it to right page }
  PropertyName:=Propertyeditor.GetName;
  PageName:=FMapPages.Values[PropertyName];
  If PageName='' then
    PageName:=FMapPages.Values['_All'];

  Index:=TabSet1.Tabs.IndexOf(Pagename);
  If (Index<>-1) then begin;
    PropertyLabel:=TLabel.Create(Self);

    PropertyLabel.AutoSize:=true;
    PropertyLabel.OnClick:=SetFocusPropertyEditor;

    VPropertyEditor:=TVpe.Create(Self);
{    VPropertyEditor.OnEditSubProperties:=SubProperties;}
    VPropertyEditor.OnActivate:=SetupHint;
    VPropertyEditor.Hint:=FHints.Values[PropertyName];
    VPropertyEditor.ParentShowHint:=true;

    {PropertyLabel.FocusControl:=VPropertyEditor};
    PropertyLabel.Tag:=longint(VPropertyEditor);

    PositionPropertyEditor(Index,PropertyLabel,VPropertyEditor,PropertyEditor);
    PropertyLabel.Caption:=PropertyEditor.GetName;
    VPropertyEditor.PropertyEditor:=PropertyEditor;
  end else begin
    PropertyEditor.Free;
  end;
end;

Function TInspector.GetActiveForm;
begin
  If FDesigner=nil then result:=nil
  else Result:=FDesigner.Form as TForm;
end;

Procedure TInspector.PositionPropertyEditor;
var
  Page : TWinControl;
begin
  { Only add this property if it is index, and add it to right page }

  if (PageIndex<>-1)then begin
    Page:=NoteBook1.Pages.Objects[PageIndex] as TWinControl;
    PropertyLabel.Parent:=Page;
    PropertyLabel.SetBounds(8,Page.Tag,GetShowWidth,PropertyLabel.Height);
    PropertyLabel.Align:=alTop;
    PropertyLabel.Visible:=true;
    Page.Tag:=Page.Tag+PropertyLabel.Height;

    VPropertyEditor.Parent:=Page;
    VPropertyEditor.SetBounds(8,Page.Tag,GetShowWidth,VPropertyEditor.height);
    VPropertyEditor.Align:=alTop;
    VPropertyEditor.Visible:=true;
    Page.Tag:=Page.Tag+VPropertyEditor.height;

    Page.Tag:=Page.Tag+8;
  end else begin
    PropertyLabel.Free;
    VPropertyEditor.Free;
  end;
end;

procedure TInspector.TabSet1Change(Sender: TObject; NewTab: Integer;
  var AllowChange: Boolean);
begin
  NoteBook1.PageIndex:=NewTab;
  {SAMFIX destroywnd the old window..??}
end;

Procedure TInspector.Prepare;

  Procedure ParsePages(Dest,Hints,Source : TStrings);
  var
    c, Index : Integer;
    hint, line, dummy, item, page : string;

  begin
    c:=0; while c<Source.Count do begin
      line:=Source[c];
      split('=',line,item,page);
      split(',',page,page,hint);
      { LHS is a property RHS is a page }
      Dummy:=Dest.Values[item];
      Index:=Dest.IndexOf(item+'='+dummy);
      if Index=-1 then
        Dest.Add(Item+'='+page)
      else begin
        Dest[Index]:=(Item+'='+page);
        Dest.Move(Index,Dest.Count-1);
      end;
      { now for hint }
      If hint<>'' then begin
        Line:=Item+'='+hint;
        Dummy:=Hints.Values[item];
        Index:=Hints.IndexOf(item+'='+dummy);
        if Index=-1 then
          Hints.Add(Line)
        else begin
          Hints[Index]:=(Line);
          hints.Move(Index,hints.Count-1);
        end;
      end;

      inc(c);
    end;
  end;

var
  CurrentClass : TClass;
  ClassList : TList;
  I : String;
  Pages : TStringList;
  c : integer;
  page, dummy : string;
begin
  ClassList:=TList.Create;
  Pages:=TStringList.Create;
  try
    CurrentClass:=TComponent(ComponentList[0]).ClassType;
    while (CurrentClass<>TObject) do begin
      ClassList.Add(CurrentClass);
      CurrentClass:=CurrentClass.ClassParent;
    end;
    { Now go through classlist from last to first and make changes }
    Pages.Clear;
    FMapPages.Clear;
    FHints.Clear;
    c:=ClassList.Count-1; while c>=0 do begin
      CurrentClass:=ClassList[c];
      Pages.Clear;
      RTDPMainForm_CSL.MapFile.ReadSectionValues(CurrentClass.Classname,Pages);
      ParsePages(FMapPages,FHints,Pages);

      dec(c);
    end;

  { Now see which pages exist at all }
  Pages.Clear;
  TabSet1.Tabs.Clear;
  c:=0; while c<FMapPages.Count do begin
    Split('=',FMapPages[c],dummy,page);
    if (page<>'-') and (TabSet1.Tabs.Indexof(Page)=-1) then
      TabSet1.tabs.Add(Page);
    inc(c);
  end;
  NoteBook1.Pages:=TabSet1.Tabs;
  if TabSet1.Tabs.Count>0 then begin
    TabSet1.TabIndex:=0;
    NoteBook1.PageIndex:=0;
  end;
  finally
    ClassList.Free;
    Pages.Free;
  end;
end;

Procedure TInspector.Clear;
var
  Component : TComponent;
  c : Integer;
  Dispose : TList;
begin
  if (csDestroying in ComponentState) then exit;
  { Roughly, disposde of any property editors, and disable text controls }

  Dispose:=TList.Create;
  try
    c:=0;
    while c<ComponentCount do begin
      Component:=Components[c];
      { Disable labels pointing to property editors }
      If Component is TLabel then
        If Component.Tag<>0 then
          Dispose.Add(Component);

      { Free associated property editors }
      If Component is TVPE then
        Dispose.Add(Component);
      inc(c);
    end;
    c:=0;
    while c<Dispose.Count do begin
      Component:=Dispose.Items[c];
      Component.Free;
      inc(c);
    end;
    NoteBook1.Pages.Clear;
    TabSet1.Tabs.Clear;
  finally
    Dispose.Free;
  end;
end;

Procedure TInspector.RemoveBlankPages;
var
  C : Integer;
  Page : TWinControl;
  Deleted : Boolean;
begin
  Deleted:=False;
  C:=0; while c<NoteBook1.Pages.Count do begin
    If (NoteBook1.Pages.Objects[c] as TWinControl).ControlCount=0 then begin
      NoteBook1.Pages.Delete(c);
      TabSet1.tabs.Delete(c);
      Deleted:=true;
    end else
      inc(c);
  end;
  { now select by hand a page }
  { presume same number of tabs as pages }
  if deleted and (Tabset1.Tabs.count>0) then begin
    If (Tabset1.TabIndex=-1) then
      tabset1.TabIndex:=0;
    NoteBook1.PageIndex:=Tabset1.TabIndex;
    { Bug in tnotebook, when you delete a page it sets pageindex to 0, but if you delete }
    { page 0, then you aree donefor as nothing happens as it thinks its already on that page }
    { This isn;t a proper fix as we miss out many things set as active control which setpage index does }
    TWinControl(NoteBook1.Pages.Objects[NoteBook1.PageIndex]).Visible:=True;
  end;
end;

Procedure TInspector.SetCurrentForm;
begin
  If Form=nil then begin
    LCurrentControl.Caption:='';
    LCurrentForm.Caption:='';
    RTDPMainForm_CSL.NotifyActiveForm(nil);
  end else begin
    RTDPMainForm_CSL.NotifyActiveForm(Form);
    LCurrentForm.Caption:=Form.Caption;
  end;
end;

Procedure TInspector.UseComponents;
begin
  If (Components=nil) then begin { they want to loose focus }
    If (Designer=FDesigner) then begin
      SetCurrentForm(nil);
      RTDPMainForm_CSL.NotifyActiveForm(nil);
      If LoadInspector.Running then
        RaiseOnThreadId(LoadInspector.Id,EAbort.Create('Form Dieing...'));
      Clear;
      LoadInspector.FComponents:=nil;
      LoadInspector.FDesigner:=nil;
      FDesigner:=nil;
      LoadInspector.FTask:=liNothing;
      If LoadInspector.Running then begin
        RaiseOnThreadId(LoadInspector.Id,EAbort.Create('Abort, restarting'));
      end;
      SelectNextForm(Designer.Form as TForm);
      { now find a non minimized editorform and setfocus it }
    end;
    exit;
  end;

  If Designer<>Fdesigner then begin
    FDesigner:=Designer;
    SetCurrentForm(Designer.Form);
  end;

  if Components.Count=0 then
    LCurrentControl.Caption:=''
  else LCurrentControl.Caption:=TComponent(Components[0]).Name+' '+NiceClassName(TComponent(Components[0]).Classtype);

  FKeyComponent:=Components[0];
  If Components.Count=0 then begin
    Components.Free;
    exit; {SAMFIX this should never happen }
  end;
  { now load map into MapPages }
  If LoadInspector.Running then begin
    RaiseOnThreadId(LoadInspector.Id,EAbort.Create('Abort, restarting'));
  end;
  LoadInspector.FComponents:=Components;
  LoadInspector.FDesigner:=Designer;
  LoadInspector.FTask:=liLoad;
  If NewThread(LoadInspector)<0 then
    LoadInspector.Run;
end;

Procedure TInspector.SelectNextForm;
var
  Form : TForm;
  C : Integer;
begin
  c:=0;
  While c<Screen.FormCount do begin
    Form:=Screen.Forms[c];
    If ((Form.designer<>nil) and not SYSMinimized(Form)) and (Form.ClassType=TForm)
    and (Form<>OldForm) then begin
      Form.BringToFront;
      break;
    end;
    inc(c);
  end;
end;

Procedure TInspector.Renew;
begin
  If FDesigner=Designer then begin
    SetCurrentForm(ActiveForm);
    LoadInspector.FNeedsRenew:=False;
  {  Image.Invalidate;}
    If LoadInspector.Running then begin
      If LoadInspector.FTask<>liRenew then
        LoadInspector.FNeedsRenew:=True
      else begin
        RaiseOnThreadId(LoadInspector.Id,EAbort.Create('Abort, restarting'));
        LoadInspector.FInspector:=Self;
        LoadInspector.FTask:=liRenew;
        If NewThread(LoadInspector)<0 then
        LoadInspector.Run;
      end;
    end else begin
      LoadInspector.FInspector:=Self;
      LoadInspector.FTask:=liRenew;
      If NewThread(LoadInspector)<0 then
      LoadInspector.Run;
    end;
  end;
end;

{Procedure TInspector.DoHeight;
begin
  Control.Top:=FHeight;
  FHeight:=FHeight+Control.Height;
end;
}
Procedure TLoadInspector.Run;
begin
  try
    Case FTask of
      liLoad: Load;
      liRenew: Renew;
    end;
  except
    On E: EAbort do raise;
    On E: Exception do begin
      ShowException(E,nil);
      Raise;
    end;
  end;
end;

Procedure TLoadInspector.Renew;
var
  c : integer;
  Component : TComponent;
  CurrentInspector : TInspector;
begin
  CurrentInspector:=FInspector;
  FNeedsRenew:=False;

 { Update values for all these property editors }
 c:=0; while c<CurrentInspector.ComponentCount do begin
   Component:=CurrentInspector.Components[c];
   If Component is TCustomVisualPropertyEditor then
     (Component as TCustomVisualPropertyEditor).Renew
   else If (Component is TVpe) then
     (Component as TVpe).Renew;
   inc(c);
   SwitchThread;
 end;
end;

Procedure TLoadInspector.Load;
var
  ComponentList : TComponentList;
  c : Integer;
begin
  If FComponents=nil then exit;
  FInspector.Enabled:=False;
  try
    FInspector.Clear;
    FInspector.Prepare(FComponents);
    SwitchThread;
    ComponentList:=TComponentList.Create;
    try
      c:=0; while c<FComponents.Count do begin
        ComponentList.Add(FComponents[c]);
        inc(c);
      end;
      GetComponentProperties(ComponentList,tkProperties,FDesigner,AddProperty);
      FInspector.RemoveBlankPages;
    finally
      ComponentList.Free;
    end;
  finally
    FComponents.Free;
    FInspector.Enabled:=True;
  end;
  If FNeedsRenew then Renew;
end;

Procedure TLoadInspector.AddProperty;
begin
  FInspector.AddProperty(PropertyEditor);
  SwitchThread;
end;

procedure TInspector.FormShow(Sender: TObject);
var
  Screenrect : TRect;
begin
{  Screenrect.TopLeft:=Point(0,0);
  ScreenRect.Right:=GetSystemMetrics(SM_CXSCREEN);
  ScreenRect.Bottom:=GetSystemMetrics(SM_CYSCREEN);
  Left:=ScreenRect.Right-Width-10;
  Top:=10;
  Height:=ScreenRect.Bottom-30;}
{  If not PtInRect(ScreenRect,Parent.ClientToScreen(Point(X,Y)) then begin

  end;}
end;

procedure TInspector.OKButtonClick(Sender: TObject);
begin
  (Sender as TBitBtn).Setfocus;
end;

Procedure TInspector.SetupHint;
begin
{  HelpBox.text:=(Sender as TVpe).Hint;}
end;

procedure TInspector.SetFocusPropertyEditor(Sender: TObject);
var
  VisualPropertyEditor : TVpe;
begin
  VisualPropertyeditor:=TVpe((Sender as TLabel).Tag);
  VisualPropertyEditor.OnClick(VisualPropertyEditor);
end;

Procedure TInspector.HideHelp;
begin
{  if HelpBox.Visible then begin
    HelpBox.Visible:=False;
    Width:=Width-HelpBox.Width;
  end;}
end;

Procedure TInspector.ShowHelp;
begin
{  if not Helpbox.Visible then begin
    HelpBox.Visible:=True;
    Width:=Width+HelpBox.Width;
  end;}
end;

procedure TInspector.FormResize(Sender: TObject);
begin
  RefreshProportions;
end;

Function TInspector.GetShowWidth;
begin
  Result:=NoteBook1.Width-16;
end;

Procedure TInspector.RefreshProportions;
var
  c : Integer;
  Component : TComponent;
  W : integer;
begin
  { make sure TVpe's are right width }
  W:=getShowWidth;
  c:=0; while c<ComponentCount do begin
    Component:=Components[c];
    If ((Component is TLabel) and (Component.Tag<>0)) or (Component is TVpe) then begin
      (Component as TControl).Width:=W;
    end;
    inc(c);
  end;
end;

procedure TInspector.CheckBox2Click(Sender: TObject);
begin
  With (Sender as TcheckBox) do If Checked then ShowHelp else HideHelp;
end;

procedure TInspector.ShowHelpCBClick(Sender: TObject);
begin
  If ((Sender as TCheckBox).Checked) then ShowHelp else HideHelp;
end;

procedure TInspector.CBOnTopClick(Sender: TObject);
begin
  If (Sender as TCheckBox).Checked then
    FormStyle:=fsStayOntop
  else
    FormStyle:=fsNormal;
end;

procedure TInspector.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Action:=caMinimize;
end;

procedure TInspector.CommitChanges;
begin
  If Designer=FDesigner then begin
    CommitEdit(commit);
  end;
end;

procedure TInspector.InspHelpButtonClick(Sender: TObject);
begin
  If not Application.HelpContext(40) then ShowMessage('Cannot show help');
end;

procedure TInspector.FormCreate(Sender: TObject);
begin
  ParentShowHint:=True;
end;

end.

